@@ -107,12 +107,14 @@ function(x, control = list())
         min(wl[2], .Machine$integer.max) else .Machine$integer.max
 
     m <- tdm(txt,
+             isTRUE(control$removePunctuation),
              isTRUE(control$removeNumbers),
              .stopwords, .dictionary,
              as.integer(min_term_freq), as.integer(max_term_freq),
              as.integer(min_word_length), as.integer(max_word_length))
 
-    m <- .SimpleTripletMatrix(m$i, m$j, m$v, enc2utf8(m$terms), x)
+    Encoding(m$terms) <- "UTF-8"
+    m <- .SimpleTripletMatrix(m$i, m$j, m$v, m$terms, x)
 
     ## Stemming
     ## <NOTE>
@@ -121,13 +123,17 @@ function(x, control = list())
     ## significant overhead (as SnowballC does not export its internal C
     ## functions).
     ##
-    ## Stemming afterwards is still quite performant as we already have all
-    ## terms. However, there is some overhead involved as we need to recheck
-    ## local bounds and word lengths.
+    ## Stemming afterwards is still quite performant as we already have
+    ## all terms.  However, there is some overhead involved as we need
+    ## to recheck local bounds and word lengths.
     ## </NOTE>
     if (isTRUE(control$stemming)) {
-        stems <- as.factor(SnowballC::wordStem(m$dimnames$Terms,
-                                               meta(x, "language")))
+        stems <- SnowballC::wordStem(m$dimnames$Terms,
+                                     meta(x, "language"))
+        ## Do as.factor(stems) "by hand" for performance reasons.
+        uniqs <- sort(unique(stems))
+        stems <- match(stems, uniqs)
+        attributes(stems) <- list(levels = uniqs, class = "factor")
         m <- rollup(m, "Terms", stems)
 
         ## Recheck local bounds
@@ -548,6 +554,9 @@ function(x, size = 10)
     stopifnot(inherits(x, c("DocumentTermMatrix", "TermDocumentMatrix")),
               is.numeric(size), size >= 0)
 
+    if (length(x$v) == 0L)
+        return(x)
+
     m <- if (inherits(x, "DocumentTermMatrix")) t(x) else x
     terms <- sort(names(sort(row_sums(m), decreasing = TRUE)
                         [0:min(size, nTerms(m))]))
